@@ -15,12 +15,6 @@ namespace fizz {
 
 using folly::AsyncSocketException;
 
-/**
- * Min and max read buffer sizes when using non-movable buffer.
- */
-static const uint32_t kMinReadSize = 1460;
-static const uint32_t kMaxReadSize = 4000;
-
 /**
  * Buffer size above which we should unset our read callback to apply back
  * pressure on the transport.
@@ -39,9 +33,7 @@ AsyncFizzBase::AsyncFizzBase(
     : folly::WriteChainAsyncTransportWrapper<folly::AsyncTransportWrapper>(
           std::move(transport)),
       handshakeTimeout_(*this, transport_->getEventBase()),
-      transportOptions_(std::move(options)),
-      ioVecQueue_(folly::IOBufIovecBuilder::Options().setBlockSize(
-          transportOptions_.readVecBlockSize)) {
+      transportOptions_(std::move(options)) {
   setReadMode(transportOptions_.readMode);
 }
 
@@ -364,7 +356,7 @@ class AsyncFizzBase::FizzMsgHdr : public folly::EventRecvmsgCallback::MsgHdr {
   iovec iov_;
 };
 
-folly::EventRecvmsgCallback::MsgHdr* AsyncFizzBase::allocateData() {
+folly::EventRecvmsgCallback::MsgHdr* AsyncFizzBase::allocateData() noexcept {
   auto* ret = msgHdr_.release();
   if (!ret) {
     ret = new FizzMsgHdr(this);
@@ -390,8 +382,9 @@ void AsyncFizzBase::eventRecvmsgCallback(FizzMsgHdr* msgHdr, int res) {
 }
 
 void AsyncFizzBase::getReadBuffer(void** bufReturn, size_t* lenReturn) {
-  std::pair<void*, uint32_t> readSpace =
-      transportReadBuf_.preallocate(kMinReadSize, kMaxReadSize);
+  std::pair<void*, uint32_t> readSpace = transportReadBuf_.preallocate(
+      transportOptions_.readBufferMinReadSize,
+      transportOptions_.readBufferAllocationSize);
   *bufReturn = readSpace.first;
 
   // `readSizeHint_`, if zero, indicates that we do not care about how much
@@ -409,21 +402,25 @@ void AsyncFizzBase::getReadBuffer(void** bufReturn, size_t* lenReturn) {
   // in WaitForData actions.
   if (readSizeHint_ > 0) {
     *lenReturn = std::min(
-        static_cast<decltype(readSizeHint_)>(kMinReadSize), readSizeHint_);
+        static_cast<decltype(readSizeHint_)>(
+            transportOptions_.readBufferMinReadSize),
+        readSizeHint_);
   } else {
     *lenReturn = readSpace.second;
   }
 }
 
 void AsyncFizzBase::getReadBuffers(folly::IOBufIovecBuilder::IoVecVec& iovs) {
-  ioVecQueue_.allocateBuffers(iovs, kMaxReadSize);
+  DCHECK(!!transportOptions_.ioVecQueue);
+  transportOptions_.ioVecQueue->allocateBuffers(iovs);
 }
 
 void AsyncFizzBase::readDataAvailable(size_t len) noexcept {
   DelayedDestruction::DestructorGuard dg(this);
 
   if (getReadMode() == folly::AsyncTransport::ReadCallback::ReadMode::ReadVec) {
-    auto tmp = ioVecQueue_.extractIOBufChain(len);
+    DCHECK(!!transportOptions_.ioVecQueue);
+    auto tmp = transportOptions_.ioVecQueue->extractIOBufChain(len);
     transportReadBuf_.append(std::move(tmp));
   } else {
     transportReadBuf_.postallocate(len);
@@ -542,6 +539,12 @@ class SecretVisitor {
       case EarlySecrets::EarlyExporter:
         callback_->earlyExporterSecretAvailable(secretBuf_);
         return;
+      case EarlySecrets::ECHAcceptConfirmation:
+        // Not an actual encryption secret
+        return;
+      case EarlySecrets::HRRECHAcceptConfirmation:
+        // Not an actual encryption secret
+        return;
     }
   }
   void operator()(const HandshakeSecrets& secret) {
