@@ -84,6 +84,14 @@ class AsyncFizzBase : public folly::WriteChainAsyncTransportWrapper<
         std::unique_ptr<folly::IOBuf> endOfData) = 0;
   };
 
+  /* Interface used to get a reference to an folly::IOBufIovecBuilder
+   */
+  struct IOVecQueueOps {
+    virtual ~IOVecQueueOps() = default;
+    virtual void allocateBuffers(folly::IOBufIovecBuilder::IoVecVec& iovs) = 0;
+    virtual std::unique_ptr<folly::IOBuf> extractIOBufChain(size_t len) = 0;
+  };
+
   struct TransportOptions {
     /**
      * Controls whether or not the async recv callback should be registered
@@ -106,17 +114,45 @@ class AsyncFizzBase : public folly::WriteChainAsyncTransportWrapper<
      *   ReadMode::ReadVec
      *      Under this mode, Fizz will use vectored IO (`readv`) to read
      *      incoming data. This can help avoid additional copies at the expense
-     *      of allocating extra ref counting objects. The overall mem usage is
-     * also dependent on the readVecBlockSize value.
+     *      of allocating extra ref counting objects.
      *
      */
     folly::AsyncReader::ReadCallback::ReadMode readMode{ReadMode::ReadBuffer};
 
     /*
-     * AsyncTransport read vec block size
+     * Under `ReadMode::ReadVec`, Fizz will allocate buffers for vectored I/O
+     * through the `iovecQueue`.
+     *
+     * This field must be set if `readMode` is `ReadMode::ReadVec`.
+     *
+     * Multiple AsyncFizzBase instances may share the underlying
+     * `IOVecQueueOps`, provided that the implementation allows it. This can be
+     * useful in limiting the amount of memory allocated across a process with
+     * many Fizz connections.
      */
-    size_t readVecBlockSize{
-        folly::IOBufIovecBuilder::Options::kDefaultBlockSize};
+    std::shared_ptr<IOVecQueueOps> ioVecQueue{};
+
+    /**
+     * Under ReadMode::ReadBuffer, whenever Fizz's available read buffer space
+     * is under `readBufferMinReadSize`, `readBufferAllocationSize` controls the
+     * size of the buffer that Fizz will allocate.
+     *
+     * Higher values will result in larger memory allocations which may
+     * negatively affect memory usage on servers with many idle connections.
+     * However, this may result in fewer syscalls and allocation calls being
+     * made on the data path.
+     */
+    size_t readBufferAllocationSize{4000};
+
+    /**
+     * Under ReadMode::ReadBuffer, whenever the underlying transport becomes
+     * available to read, Fizz will ensure that *at least*
+     * `readBuferMinReadSize` worth of contiguous data is read per read call. If
+     * the internal read buffer is smaller than `readBufferMinReadSize`, then
+     * Fizz will allocate `readBufferAllocationSize` worth of new buffer space
+     * to satisfy this read.
+     */
+    size_t readBufferMinReadSize{1460};
   };
 
   explicit AsyncFizzBase(
@@ -448,7 +484,7 @@ class AsyncFizzBase : public folly::WriteChainAsyncTransportWrapper<
   /**
    * EventRecvmsgCallback implementation
    */
-  folly::EventRecvmsgCallback::MsgHdr* allocateData() override;
+  folly::EventRecvmsgCallback::MsgHdr* allocateData() noexcept override;
   void eventRecvmsgCallback(FizzMsgHdr* msgHdr, int res);
 
   /**
@@ -492,7 +528,5 @@ class AsyncFizzBase : public folly::WriteChainAsyncTransportWrapper<
 
   TransportOptions transportOptions_;
   std::unique_ptr<FizzMsgHdr> msgHdr_;
-
-  folly::IOBufIovecBuilder ioVecQueue_;
 };
 } // namespace fizz
