@@ -60,9 +60,9 @@ void checkDecodedChlo(ClientHello decodedChlo, ClientHello expectedChlo) {
 ECHConfig getInvalidVECHConfig() {
   // Add invalid config
   ECHConfig invalidConfig;
-  invalidConfig.version = ECHVersion::Draft9;
+  invalidConfig.version = ECHVersion::Draft15;
   auto configContent = getECHConfigContent();
-  configContent.kem_id = hpke::KEMId::secp384r1;
+  configContent.key_config.kem_id = hpke::KEMId::secp384r1;
   invalidConfig.ech_config_content = encode(std::move(configContent));
 
   return invalidConfig;
@@ -78,10 +78,13 @@ hpke::SetupResult constructSetupResult(
   return constructHpkeSetupResult(std::move(kex), supportedConfig);
 }
 
-ClientECH getTestClientECHWithInner(ClientHello chloInner) {
+OuterECHClientHello getTestOuterECHClientHelloWithInner(ClientHello chloInner) {
+  auto configContent = getECHConfigContent();
   SupportedECHConfig supportedConfig{
       getECHConfig(),
-      ECHCipherSuite{
+      configContent.key_config.config_id,
+      configContent.maximum_name_length,
+      HpkeSymmetricCipherSuite{
           hpke::KDFId::Sha256, hpke::AeadId::TLS_AES_128_GCM_SHA256}};
 
   auto setupResult = constructSetupResult(supportedConfig);
@@ -91,11 +94,12 @@ ClientECH getTestClientECHWithInner(ClientHello chloInner) {
       supportedConfig,
       std::move(chloInner),
       getClientHelloOuter(),
-      setupResult);
+      setupResult,
+      folly::none);
 }
 
-ClientECH getTestClientECH() {
-  return getTestClientECHWithInner(TestMessages::clientHello());
+OuterECHClientHello getTestOuterECHClientHello() {
+  return getTestOuterECHClientHelloWithInner(TestMessages::clientHello());
 }
 
 void checkSupportedConfigValid(
@@ -111,6 +115,8 @@ void checkSupportedConfigValid(
   EXPECT_EQ(result.value().cipherSuite.kdf_id, hpke::KDFId::Sha256);
   EXPECT_EQ(
       result.value().cipherSuite.aead_id, hpke::AeadId::TLS_AES_128_GCM_SHA256);
+  EXPECT_EQ(result.value().configId, 0xFB);
+  EXPECT_EQ(result.value().maxLen, 100);
 }
 
 } // namespace
@@ -118,10 +124,10 @@ void checkSupportedConfigValid(
 TEST(EncryptionTest, TestValidECHConfigContent) {
   // Add config that doesn't work and cannot be supported
   ECHConfigContentDraft invalidConfigContent = getECHConfigContent();
-  invalidConfigContent.kem_id = hpke::KEMId::secp521r1;
+  invalidConfigContent.key_config.kem_id = hpke::KEMId::secp521r1;
   std::vector<ECHConfig> configs;
   ECHConfig invalid;
-  invalid.version = ECHVersion::Draft9;
+  invalid.version = ECHVersion::Draft15;
   invalid.ech_config_content = encode(std::move(invalidConfigContent));
 
   // Add config that works and can be supported
@@ -136,11 +142,11 @@ TEST(EncryptionTest, TestValidECHConfigContent) {
 TEST(EncryptionTest, TestInvalidECHConfigContent) {
   ECHConfigContentDraft configContent = getECHConfigContent();
 
-  configContent.kem_id = hpke::KEMId::secp256r1;
-  ECHCipherSuite suite{
+  configContent.key_config.kem_id = hpke::KEMId::secp256r1;
+  HpkeSymmetricCipherSuite suite{
       hpke::KDFId::Sha512, hpke::AeadId::TLS_AES_128_GCM_SHA256};
-  std::vector<ECHCipherSuite> cipher_suites = {suite};
-  configContent.cipher_suites = cipher_suites;
+  std::vector<HpkeSymmetricCipherSuite> cipher_suites = {suite};
+  configContent.key_config.cipher_suites = cipher_suites;
 
   ECHConfig invalidConfig;
   invalidConfig.version = static_cast<ECHVersion>(0xfe07); // Draft 7
@@ -169,7 +175,7 @@ TEST(EncryptionTest, TestUnsupportedMandatoryExtension) {
 
   std::vector<ECHConfig> configs;
   ECHConfig invalid;
-  invalid.version = ECHVersion::Draft9;
+  invalid.version = ECHVersion::Draft15;
   invalid.ech_config_content = encode(std::move(invalidConfigContent));
   configs.push_back(std::move(invalid));
 
@@ -183,7 +189,7 @@ TEST(EncryptionTest, TestUnsupportedMandatoryExtension) {
 TEST(EncryptionTest, TestValidSelectECHConfigContent) {
   // Add valid config
   ECHConfig validConfig;
-  validConfig.version = ECHVersion::Draft9;
+  validConfig.version = ECHVersion::Draft15;
   validConfig.ech_config_content = encode(getECHConfigContent());
 
   std::vector<ECHConfig> configs;
@@ -204,10 +210,10 @@ TEST(EncryptionTest, TestInvalidSelectECHConfigContent) {
 }
 
 TEST(EncryptionTest, TestValidEncryptClientHello) {
-  auto clientECH = getTestClientECH();
+  auto clientECH = getTestOuterECHClientHello();
   auto expectedChlo = TestMessages::clientHello();
   // Add a legacy_session_id to match client hello inner used in
-  // getTestClientECH()
+  // getTestOuterECHClientHello()
   expectedChlo.legacy_session_id =
       folly::IOBuf::copyBuffer("test legacy session id");
 
@@ -218,7 +224,7 @@ TEST(EncryptionTest, TestValidEncryptClientHello) {
   hpkePrefix->prependChain(encode(getECHConfig()));
 
   const std::unique_ptr<folly::IOBuf> prefix =
-      folly::IOBuf::copyBuffer("HPKE-07");
+      folly::IOBuf::copyBuffer("HPKE-v1");
 
   auto kex = std::make_unique<MockOpenSSLECKeyExchange256>();
   kex->setPrivateKey(getPrivateKey(kP256Key));
@@ -245,23 +251,37 @@ TEST(EncryptionTest, TestValidEncryptClientHello) {
   auto context = setupWithDecap(
       hpke::Mode::Base,
       clientECH.enc->coalesce(),
+      folly::none,
       std::move(hpkePrefix),
       folly::none,
       std::move(setupParam));
 
   // Get client hello inner by decrypting
-  auto clientHelloOuterEnc = encode(getClientHelloOuter());
-  auto clientHelloOuterAad = makeClientHelloAad(
-      clientECH.cipher_suite,
-      clientECH.config_id,
-      clientECH.enc,
-      clientHelloOuterEnc);
+  auto clientHelloOuter = getClientHelloOuter();
+  auto dummyECH = getTestOuterECHClientHello();
+  dummyECH.payload->coalesce();
+  memset(dummyECH.payload->writableData(), 0, dummyECH.payload->length());
+  clientHelloOuter.extensions.push_back(encodeExtension(dummyECH));
+
+  auto clientHelloOuterAad = encode(clientHelloOuter);
+
   std::unique_ptr<folly::IOBuf> gotClientHelloInner =
       context->open(clientHelloOuterAad.get(), std::move(clientECH.payload));
 
   folly::io::Cursor encodedECHInnerCursor(gotClientHelloInner.get());
   auto gotChlo = decode<ClientHello>(encodedECHInnerCursor);
 
+  // Check padding
+  auto configContent = getECHConfigContent();
+  auto paddingSize = calculateECHPadding(
+      gotChlo,
+      encodedECHInnerCursor.getCurrentPosition(),
+      configContent.maximum_name_length);
+  for (size_t i = 0; i < paddingSize; i++) {
+    EXPECT_EQ(0, encodedECHInnerCursor.read<uint8_t>());
+  }
+  EXPECT_TRUE(encodedECHInnerCursor.isAtEnd());
+
   // Check that we don't have a legacy_session_id (it should have gotten removed
   // during encryption)
   EXPECT_TRUE(folly::IOBufEqualTo()(
@@ -280,7 +300,7 @@ TEST(EncryptionTest, TestTryToDecryptECH) {
 
   // Add ECH extension to client hello outer.
   auto chloOuter = getClientHelloOuter();
-  auto testECH = getTestClientECH();
+  auto testECH = getTestOuterECHClientHello();
   chloOuter.extensions.push_back(encodeExtension(testECH));
 
   auto kex = std::make_unique<MockOpenSSLECKeyExchange256>();
@@ -301,7 +321,7 @@ TEST(EncryptionTest, TestTryToDecryptECH) {
       std::move(testECH.enc),
       std::move(testECH.config_id),
       std::move(testECH.payload),
-      ECHVersion::Draft9,
+      ECHVersion::Draft15,
       context);
 
   checkDecodedChlo(std::move(chlo), std::move(expectedChlo));
@@ -313,7 +333,7 @@ TEST(EncryptionTest, TestInnerClientHelloOuterExtensionsContainsECH) {
   OuterExtensions outer;
   outer.types = {ExtensionType::encrypted_client_hello};
   innerChlo.extensions.push_back(encodeExtension(std::move(outer)));
-  auto clientECH = getTestClientECHWithInner(std::move(innerChlo));
+  auto clientECH = getTestOuterECHClientHelloWithInner(std::move(innerChlo));
 
   // Create HPKE setup prefix
   std::string tlsEchPrefix = "tls ech";
@@ -322,7 +342,7 @@ TEST(EncryptionTest, TestInnerClientHelloOuterExtensionsContainsECH) {
   hpkePrefix->prependChain(encode(getECHConfig()));
 
   const std::unique_ptr<folly::IOBuf> prefix =
-      folly::IOBuf::copyBuffer("HPKE-07");
+      folly::IOBuf::copyBuffer("HPKE-v1");
 
   auto kex = std::make_unique<MockOpenSSLECKeyExchange256>();
   kex->setPrivateKey(getPrivateKey(kP256Key));
@@ -349,6 +369,7 @@ TEST(EncryptionTest, TestInnerClientHelloOuterExtensionsContainsECH) {
   auto context = setupWithDecap(
       hpke::Mode::Base,
       clientECH.enc->coalesce(),
+      folly::none,
       std::move(hpkePrefix),
       folly::none,
       std::move(setupParam));
@@ -363,7 +384,7 @@ TEST(EncryptionTest, TestInnerClientHelloOuterExtensionsContainsECH) {
           std::move(clientECH.enc),
           std::move(clientECH.config_id),
           std::move(clientECH.payload),
-          ECHVersion::Draft9,
+          ECHVersion::Draft15,
           context),
       OuterExtensionsError);
 }
@@ -374,7 +395,7 @@ TEST(EncryptionTest, TestInnerClientHelloOuterExtensionsContainsDupes) {
   OuterExtensions outer;
   outer.types = {ExtensionType::server_name};
   innerChlo.extensions.push_back(encodeExtension(std::move(outer)));
-  auto clientECH = getTestClientECHWithInner(std::move(innerChlo));
+  auto clientECH = getTestOuterECHClientHelloWithInner(std::move(innerChlo));
 
   // Create HPKE setup prefix
   std::string tlsEchPrefix = "tls ech";
@@ -383,7 +404,7 @@ TEST(EncryptionTest, TestInnerClientHelloOuterExtensionsContainsDupes) {
   hpkePrefix->prependChain(encode(getECHConfig()));
 
   const std::unique_ptr<folly::IOBuf> prefix =
-      folly::IOBuf::copyBuffer("HPKE-07");
+      folly::IOBuf::copyBuffer("HPKE-v1");
 
   auto kex = std::make_unique<MockOpenSSLECKeyExchange256>();
   kex->setPrivateKey(getPrivateKey(kP256Key));
@@ -410,6 +431,7 @@ TEST(EncryptionTest, TestInnerClientHelloOuterExtensionsContainsDupes) {
   auto context = setupWithDecap(
       hpke::Mode::Base,
       clientECH.enc->coalesce(),
+      folly::none,
       std::move(hpkePrefix),
       folly::none,
       std::move(setupParam));
@@ -424,7 +446,7 @@ TEST(EncryptionTest, TestInnerClientHelloOuterExtensionsContainsDupes) {
           std::move(clientECH.enc),
           std::move(clientECH.config_id),
           std::move(clientECH.payload),
-          ECHVersion::Draft9,
+          ECHVersion::Draft15,
           context),
       OuterExtensionsError);
 }
@@ -650,6 +672,232 @@ TEST(EncryptionTest, TestSubstituteOuterExtensions) {
       OuterExtensionsError);
 }
 
+TEST(EncryptionTest, TestMakeDummyServerHello) {
+  auto shlo = TestMessages::serverHello();
+  auto dummyShlo = makeDummyServerHello(shlo);
+  EXPECT_EQ(shlo.legacy_version, dummyShlo.legacy_version);
+  // Check that the non-ECH part of the random is preserved.
+  auto randomUnmodifiedRange = folly::Range(
+      shlo.random.begin(), shlo.random.end() - kEchAcceptConfirmationSize);
+  auto dummyRandomUnmodifiedRange = folly::Range(
+      dummyShlo.random.begin(),
+      dummyShlo.random.end() - kEchAcceptConfirmationSize);
+  EXPECT_EQ(randomUnmodifiedRange, dummyRandomUnmodifiedRange);
+
+  // Check that the end is zeroes.
+  for (auto it = dummyShlo.random.end() - kEchAcceptConfirmationSize;
+       it != dummyShlo.random.end();
+       it++) {
+    EXPECT_EQ(*it, 0);
+  }
+
+  EXPECT_TRUE(folly::IOBufEqualTo()(
+      shlo.legacy_session_id_echo, dummyShlo.legacy_session_id_echo));
+  EXPECT_EQ(shlo.cipher_suite, dummyShlo.cipher_suite);
+  EXPECT_EQ(
+      shlo.legacy_compression_method, dummyShlo.legacy_compression_method);
+  EXPECT_EQ(shlo.extensions.size(), dummyShlo.extensions.size());
+  for (size_t i = 0; i < shlo.extensions.size(); i++) {
+    EXPECT_EQ(
+        shlo.extensions[i].extension_type,
+        dummyShlo.extensions[i].extension_type);
+    EXPECT_TRUE(folly::IOBufEqualTo()(
+        shlo.extensions[i].extension_data,
+        dummyShlo.extensions[i].extension_data));
+  }
+}
+
+namespace {
+
+Extension makeDummyECHExtension() {
+  auto buf = folly::IOBuf::create(kEchAcceptConfirmationSize);
+  memset(buf->writableData(), 0xAC, kEchAcceptConfirmationSize);
+  buf->append(kEchAcceptConfirmationSize);
+  return {ExtensionType::encrypted_client_hello, std::move(buf)};
+}
+
+} // namespace
+
+TEST(EncryptionTest, TestMakeDummyHRR) {
+  auto hrr = TestMessages::helloRetryRequest();
+  hrr.extensions.push_back(makeDummyECHExtension());
+  auto dummyHrr = makeDummyHRR(hrr);
+  EXPECT_EQ(hrr.legacy_version, dummyHrr.legacy_version);
+  EXPECT_TRUE(folly::IOBufEqualTo()(
+      hrr.legacy_session_id_echo, dummyHrr.legacy_session_id_echo));
+  EXPECT_EQ(hrr.cipher_suite, dummyHrr.cipher_suite);
+  EXPECT_EQ(hrr.legacy_compression_method, dummyHrr.legacy_compression_method);
+  EXPECT_EQ(hrr.extensions.size(), dummyHrr.extensions.size());
+  for (size_t i = 0; i < hrr.extensions.size(); i++) {
+    EXPECT_EQ(
+        hrr.extensions[i].extension_type,
+        dummyHrr.extensions[i].extension_type);
+    // If extension type is ech, expect it to be zeroed.
+    if (hrr.extensions[i].extension_type ==
+        ExtensionType::encrypted_client_hello) {
+      EXPECT_EQ(
+          dummyHrr.extensions[i].extension_data->computeChainDataLength(),
+          hrr.extensions[i].extension_data->computeChainDataLength());
+      auto dummyECH = dummyHrr.extensions[i].extension_data->coalesce();
+      EXPECT_TRUE(
+          std::all_of(dummyECH.begin(), dummyECH.end(), [](const auto& c) {
+            return c == 0;
+          }));
+    } else {
+      EXPECT_TRUE(folly::IOBufEqualTo()(
+          hrr.extensions[i].extension_data,
+          dummyHrr.extensions[i].extension_data));
+    }
+  }
+}
+
+MATCHER_P(DummyShloMatch, shlo, "") {
+  return folly::IOBufEqualTo()(
+      encodeHandshake(makeDummyServerHello(*shlo)), arg);
+}
+
+TEST(EncryptionTest, TestSetShloAcceptance) {
+  auto shlo = TestMessages::serverHello();
+  auto context = std::make_unique<MockHandshakeContext>();
+  auto scheduler = std::make_unique<MockKeyScheduler>();
+  auto acceptSecret = std::vector<uint8_t>(kEchAcceptConfirmationSize, 0xAC);
+  EXPECT_CALL(*context, appendToTranscript(DummyShloMatch(&shlo)));
+  EXPECT_CALL(*context, getHandshakeContext()).WillOnce(Invoke([]() {
+    return folly::IOBuf::copyBuffer("dummyshlo");
+  }));
+  EXPECT_CALL(
+      *scheduler,
+      getSecret(EarlySecrets::ECHAcceptConfirmation, RangeMatches("dummyshlo")))
+      .WillOnce(InvokeWithoutArgs([&]() {
+        return DerivedSecret(acceptSecret, EarlySecrets::ECHAcceptConfirmation);
+      }));
+  std::unique_ptr<KeyScheduler> schedulerBasePtr = std::move(scheduler);
+  setAcceptConfirmation(shlo, std::move(context), std::move(schedulerBasePtr));
+  auto shloIt = shlo.random.end() - kEchAcceptConfirmationSize;
+  auto secretIt = acceptSecret.begin();
+  for (size_t i = 0; i < kEchAcceptConfirmationSize; i++) {
+    EXPECT_EQ(*shloIt, *secretIt);
+  }
+}
+
+MATCHER_P(DummyHrrMatch, hrr, "") {
+  return folly::IOBufEqualTo()(encodeHandshake(makeDummyHRR(*hrr)), arg);
+}
+
+TEST(EncryptionTest, TestSetHRRAcceptance) {
+  auto hrr = TestMessages::helloRetryRequest();
+  auto context = std::make_unique<MockHandshakeContext>();
+  auto scheduler = std::make_unique<MockKeyScheduler>();
+  auto acceptSecret = std::vector<uint8_t>(kEchAcceptConfirmationSize, 0xAC);
+  EXPECT_CALL(*context, appendToTranscript(DummyHrrMatch(&hrr)));
+  EXPECT_CALL(*context, getHandshakeContext()).WillOnce(Invoke([]() {
+    return folly::IOBuf::copyBuffer("dummyhrr");
+  }));
+  EXPECT_CALL(
+      *scheduler,
+      getSecret(
+          EarlySecrets::HRRECHAcceptConfirmation, RangeMatches("dummyhrr")))
+      .WillOnce(InvokeWithoutArgs([&]() {
+        return DerivedSecret(
+            acceptSecret, EarlySecrets::HRRECHAcceptConfirmation);
+      }));
+  std::unique_ptr<KeyScheduler> schedulerBasePtr = std::move(scheduler);
+  setAcceptConfirmation(hrr, std::move(context), std::move(schedulerBasePtr));
+  auto echExtensionRange = hrr.extensions.back().extension_data->coalesce();
+  auto echExtensionIt = echExtensionRange.begin();
+  auto secretIt = acceptSecret.begin();
+  for (size_t i = 0; i < kEchAcceptConfirmationSize; i++) {
+    EXPECT_EQ(*echExtensionIt, *secretIt);
+  }
+}
+
+TEST(EncryptionTest, TestCheckShloAcceptance) {
+  auto shlo = TestMessages::serverHello();
+  auto context = std::make_unique<MockHandshakeContext>();
+  auto scheduler = std::make_unique<MockKeyScheduler>();
+  auto acceptSecret = std::vector<uint8_t>(kEchAcceptConfirmationSize, 0xAC);
+  std::fill(
+      shlo.random.end() - kEchAcceptConfirmationSize, shlo.random.end(), 0xAC);
+  EXPECT_CALL(*context, appendToTranscript(DummyShloMatch(&shlo)));
+  EXPECT_CALL(*context, getHandshakeContext()).WillOnce(Invoke([]() {
+    return folly::IOBuf::copyBuffer("dummyshlo");
+  }));
+  EXPECT_CALL(
+      *scheduler,
+      getSecret(EarlySecrets::ECHAcceptConfirmation, RangeMatches("dummyshlo")))
+      .WillOnce(InvokeWithoutArgs([&]() {
+        return DerivedSecret(acceptSecret, EarlySecrets::ECHAcceptConfirmation);
+      }));
+  std::unique_ptr<KeyScheduler> schedulerBasePtr = std::move(scheduler);
+  EXPECT_TRUE(
+      checkECHAccepted(shlo, std::move(context), std::move(schedulerBasePtr)));
+}
+
+TEST(EncryptionTest, TestCheckHrrAcceptance) {
+  auto hrr = TestMessages::helloRetryRequest();
+  hrr.extensions.push_back(makeDummyECHExtension());
+  auto context = std::make_unique<MockHandshakeContext>();
+  auto scheduler = std::make_unique<MockKeyScheduler>();
+  auto acceptSecret = std::vector<uint8_t>(kEchAcceptConfirmationSize, 0xAC);
+  EXPECT_CALL(*context, appendToTranscript(DummyHrrMatch(&hrr)));
+  EXPECT_CALL(*context, getHandshakeContext()).WillOnce(Invoke([]() {
+    return folly::IOBuf::copyBuffer("dummyhrr");
+  }));
+  EXPECT_CALL(
+      *scheduler,
+      getSecret(
+          EarlySecrets::HRRECHAcceptConfirmation, RangeMatches("dummyhrr")))
+      .WillOnce(InvokeWithoutArgs([&]() {
+        return DerivedSecret(
+            acceptSecret, EarlySecrets::HRRECHAcceptConfirmation);
+      }));
+  std::unique_ptr<KeyScheduler> schedulerBasePtr = std::move(scheduler);
+  EXPECT_TRUE(
+      checkECHAccepted(hrr, std::move(context), std::move(schedulerBasePtr)));
+}
+
+TEST(EncryptionTest, TestGenerateGreasePsk) {
+  MockFactory factory;
+  factory.setDefaults();
+
+  // If no psk extension is present, expect no GREASE PSK.
+  EXPECT_EQ(
+      generateGreasePSK(TestMessages::clientHello(), &factory), folly::none);
+
+  auto chlo = TestMessages::clientHelloPsk();
+  auto psk = getExtension<ClientPresharedKey>(chlo.extensions);
+
+  // Check that data is replaced with random data for GREASE PSK
+  auto greasePsk = generateGreasePSK(chlo, &factory);
+  EXPECT_TRUE(greasePsk.has_value());
+  EXPECT_EQ(greasePsk->identities.size(), psk->identities.size());
+  EXPECT_EQ(greasePsk->binders.size(), psk->binders.size());
+  for (size_t i = 0; i < greasePsk->identities.size(); i++) {
+    auto idSz = psk->identities[i].psk_identity->computeChainDataLength();
+    auto binderSz = psk->binders[i].binder->computeChainDataLength();
+    auto randomId = folly::IOBuf::copyBuffer(std::string(idSz, 0x44));
+    auto randomBinder = folly::IOBuf::copyBuffer(std::string(binderSz, 0x44));
+    EXPECT_TRUE(
+        folly::IOBufEqualTo()(greasePsk->identities[i].psk_identity, randomId));
+    EXPECT_TRUE(
+        folly::IOBufEqualTo()(greasePsk->binders[i].binder, randomBinder));
+  }
+
+  // Check HRR GREASE. Same as above, but preserves identities.
+  auto hrrGreasePsk = generateGreasePSKForHRR(*psk, &factory);
+  EXPECT_EQ(hrrGreasePsk.identities.size(), psk->identities.size());
+  EXPECT_EQ(hrrGreasePsk.binders.size(), psk->binders.size());
+  for (size_t i = 0; i < hrrGreasePsk.identities.size(); i++) {
+    auto binderSz = psk->binders[i].binder->computeChainDataLength();
+    auto randomBinder = folly::IOBuf::copyBuffer(std::string(binderSz, 0x44));
+    EXPECT_TRUE(folly::IOBufEqualTo()(
+        hrrGreasePsk.identities[i].psk_identity,
+        psk->identities[i].psk_identity));
+    EXPECT_TRUE(
+        folly::IOBufEqualTo()(hrrGreasePsk.binders[i].binder, randomBinder));
+  }
+}
+
 } // namespace test
 } // namespace ech
 } // namespace fizz
