@@ -343,6 +343,16 @@ void AsyncFizzServerT<SM>::ActionMoveVisitor::operator()(
 template <typename SM>
 void AsyncFizzServerT<SM>::ActionMoveVisitor::operator()(
     SecretAvailable& secret) {
+#if 0
+  FOLLY_SDT(
+      fizz,
+      fizz_secret_available,
+      secret.secret.secret.size(),
+      secret.secret.secret.data(),
+      KeyLogWriter::secretToNSSLabel(secret.secret.type)
+          .value_or(std::numeric_limits<KeyLogWriter::Label>::max()),
+      server_.getClientRandom()->data());
+#endif
   server_.secretAvailable(secret.secret);
 }
 
